/* ----------------------------------------------------------------------------
 * ergoDOX : controller specific exports
 * ----------------------------------------------------------------------------
 * Copyright (c) 2012 Ben Blazak <benblazak.dev@gmail.com>
 * Released under The MIT License (MIT) (see "license.md")
 * Project located at <https://github.com/benblazak/ergodox-firmware>
 * ------------------------------------------------------------------------- */

#pragma once

#define USB_DEBUG_HID

#include <stdarg.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <util/twi.h>

// --------------------------------------------------------------------

/*
 * DRIVE_ROWS and DRIVE_COLUMNS
 * - Select which pins will drive (alternate between hi-Z and drive
 *   low) and which will be inputs
 *
 * Notes
 * - You must set exactly one of each 'TEENSY' macro, and of each
 *   'MCP23018' macro
 * - If you are using internal diodes (inside the key switches)... then
 *   i don't know what to tell you.  You will set one chip to drive
 *   rows, and the other to drive columns, but i don't have a key
 *   switch to check which at the moment, and i couldn't seem to find
 *   it online.
 * - If the diode cathode is towards the square solder pad, set
 *   #define  TEENSY__DRIVE_COLUMNS    1
 *   #define  MCP23018__DRIVE_COLUMNS  1
 * - If the diode cathode is towards the circular solder pad, set
 *   #define  TEENSY__DRIVE_ROWS       1
 *   #define  MCP23018__DRIVE_ROWS     1
 */
#define  TEENSY__DRIVE_ROWS     	0
#define  MCP23018__DRIVE_ROWS   	0
#define  TEENSY__DRIVE_COLUMNS  	1
#define  MCP23018__DRIVE_COLUMNS	1

#define KB_ROWS   	6 	// must match real life
#define KB_COLUMNS	14	// must match real life

// --------------------------------------------------------------------

u8 kb_init(void);
u8 kb_update_matrix(bool matrix[KB_ROWS][KB_COLUMNS]);

// --------------------------------------------------------------------

#define MCP23018_TWI_ADDRESS 0b0100000

u8 mcp23018_init(void);
u8 mcp23018_update_matrix( bool matrix[KB_ROWS][KB_COLUMNS] );

// --------------------------------------------------------------------

u8 teensy_init(void);
u8 teensy_update_matrix( bool matrix[KB_ROWS][KB_COLUMNS] );

// --------------------------------------------------------------------

#define TWI_FREQ 400000

void	twi_init  (void);
u8  	twi_start (void);
void	twi_stop  (void);
u8  	twi_send  (u8 data);
u8  	twi_read  (u8 * data);

// --------------------------------------------------------------------

void	usb_init(void);      	// initialize everything
u8  	usb_configured(void);	// is the USB port configured

i8 usb_keyboard_send(void);
extern u8 keyboard_modifier_keys;
extern u8 keyboard_keys[6];

extern u16 consumer_key;

// Everything below this point is only intended for usb_serial.c

#define EP_TYPE_CONTROL        	0x00
#define EP_TYPE_BULK_IN        	0x81
#define EP_TYPE_BULK_OUT       	0x80
#define EP_TYPE_INTERRUPT_IN   	0xC1
#define EP_TYPE_INTERRUPT_OUT  	0xC0
#define EP_TYPE_ISOCHRONOUS_IN 	0x41
#define EP_TYPE_ISOCHRONOUS_OUT	0x40

#define EP_SINGLE_BUFFER	0x02
#define EP_DOUBLE_BUFFER	0x06

#define EP_SIZE(s)	((s) == 64 ? 0x30 :         \
                     ((s) == 32 ? 0x20 :        \
                      ((s) == 16 ? 0x10 :       \
                       0x00)))

#define MAX_ENDPOINT	4

#define LSB(n) (n & 255)
#define MSB(n) ((n >> 8) & 255)

#define HW_CONFIG() 	(UHWCON = 0x01)
#define PLL_CONFIG()	(PLLCSR = 0x12)
#define USB_CONFIG()	(USBCON = ((1<<USBE)|(1<<OTGPADE)))
#define USB_FREEZE()	(USBCON = ((1<<USBE)|(1<<FRZCLK)))

// standard control endpoint request types
#define GET_STATUS       	0
#define CLEAR_FEATURE    	1
#define SET_FEATURE      	3
#define SET_ADDRESS      	5
#define GET_DESCRIPTOR   	6
#define GET_CONFIGURATION	8
#define SET_CONFIGURATION	9
#define GET_INTERFACE    	10
#define SET_INTERFACE    	11
// HID (human interface device)
#define HID_GET_REPORT  	1
#define HID_GET_IDLE    	2
#define HID_GET_PROTOCOL	3
#define HID_SET_REPORT  	9
#define HID_SET_IDLE    	10
#define HID_SET_PROTOCOL	11
// CDC (communication class device)
#define CDC_SET_LINE_CODING       	0x20
#define CDC_GET_LINE_CODING       	0x21
#define CDC_SET_CONTROL_LINE_STATE	0x22

// -----------------------------------------------------------------
// debug
// -----------------------------------------------------------------

extern volatile u8 debug_flush_timer;

void usb_debug_free_memory(void);
void usb_debug_print(const char *s);
void usb_debug_printf(const char *fmt, ...);
i8 usb_debug_putchar(char c);
void usb_debug_flush_output(void);

#ifdef KBD_DEBUG
#define debug_print(s)          	usb_debug_print(PSTR(s))
#define debug_printf(s, args...)	usb_debug_printf(PSTR(s), args)
#else
#define debug_print(s)
#define debug_printf(...)
#endif

// -----------------------------------------------------------------
// timer
// -----------------------------------------------------------------

extern volatile uint32_t timer0_ms;

void timer0_init(void);
